==> ovos_audio_transformer_plugin_ggwave-0.0.0a14-py3-none-any.whl: ovos_audio_transformer_plugin_ggwave/__init__.py <==
import datetime
from distutils.spawn import find_executable
from os.path import isfile, expanduser

import pexpect
from ovos_config import Configuration
from ovos_plugin_manager.templates.transformers import AudioTransformer
from ovos_utils import create_daemon
from ovos_utils.log import LOG
from ovos_utils.messagebus import Message
from ovos_workshop.app import OVOSAbstractApplication
from ovos_workshop.decorators import intent_handler


class GGWaveSkill(OVOSAbstractApplication):

    def initialize(self):
        self.add_event("ggwave.enabled", self.handle_ggwave_on)
        self.add_event("ggwave.disabled", self.handle_ggwave_off)
        self.enabled = False

    def handle_ggwave_on(self, message):
        self.enabled = True
        self.schedule_event(handler=self.handle_ggwave_off,
                            when=datetime.datetime.now() + datetime.timedelta(minutes=15),
                            name="ggwave.timeout")

    def handle_ggwave_off(self, message):
        self.enabled = False

    @intent_handler("enable.ggwave.intent")
    def handle_enable_ggwave(self, message):
        if not self.enabled:
            self.bus.emit(message.forward("ovos.ggwave.enable"))
            self.speak_dialog("ggwave.enabled")
        else:
            self.speak_dialog("ggwave.already.enabled")

    @intent_handler("disable.ggwave.intent")
    def handle_disable_ggwave(self, message):
        if self.enabled:
            self.bus.emit(message.forward("ovos.ggwave.disable"))
            self.cancel_scheduled_event("ggwave.timeout")
            self.speak_dialog("ggwave.disabled")
        else:
            self.speak_dialog("ggwave.already.disabled")


# NOTE - could not get ggwave to work properly with the audio feed
# ran out of time so just used a subprocess
class GGWavePlugin(AudioTransformer):

    def __init__(self, config=None):
        config = config or {}
        super().__init__("ovos-audio-transformer-plugin-ggwave", 10, config)
        self.binpath = self.config.get("binary") or \
                       find_executable("ggwave-rx") or \
                       expanduser("~/.local/bin/ggwave-rx")
        if not isfile(self.binpath):
            raise ValueError(f"ggwave-rx not found in {self.binpath}, "
                             f"please install from https://github.com/ggerganov/ggwave")
        # TODO - individual config to enable/disable each
        self.OPCODES = {
            "SSID:": self.handle_wifi_ssid,
            "PSWD:": self.handle_wifi_pswd,
            "UTT:": self.handle_utt,
            "SPEAK:": self.handle_speak,
            "JSON:": self.handle_json,
            "BUS:": self.handle_bus,
            "GHS:": self.handle_skill,
            "PIP:": self.handle_pip,
            "RMPIP:": self.handle_remove_pip
        }
        self.debug = self.config.get("debug")
        self._ssid = None
        self.vui = None
        # TODO - autoenable if wifi has not been setup previously
        self.user_enabled = self.config.get("start_enabled")

    def bind(self, bus=None):
        """ attach messagebus """
        super().bind(bus)
        # we load the voice interface as part of this plugin
        # the skill interacts only via messagebus
        self.vui = GGWaveSkill(bus=self.bus, skill_id="ggwave.openvoiceos")
        self.bus.on("ovos.ggwave.enable", self.handle_enable)
        self.bus.on("ovos.ggwave.disable", self.handle_disable)

        self.daemon = create_daemon(self.monitor_thread)

    def handle_enable(self, message: Message):
        self.user_enabled = True
        self.bus.emit(message.forward("ovos.ggwave.enabled"))
        # TODO - dedicated sound
        self.bus.emit(Message("mycroft.audio.play_sound",
                              {"uri": "snd/acknowledge.mp3"}))

    def handle_disable(self, message: Message):
        self.user_enabled = False
        self.bus.emit(message.forward("ovos.ggwave.disabled"))
        # TODO - dedicated sound
        self.bus.emit(Message("mycroft.audio.play_sound",
                              {"uri": "snd/acknowledge.mp3"}))

    def shutdown(self):
        if self.vui is not None:
            self.vui.shutdown()

    def handle_skill(self, payload):
        if not payload.startswith("https://github.com/"):
            payload = f"https://github.com/{payload}"
        LOG.info(f"github skill to install: {payload}")
        self.bus.emit(Message("ovos.skills.install", {"url": payload}))

    def handle_pip(self, payload):
        LOG.info(f"pip package to install: {payload}")
        self.bus.emit(Message("ovos.pip.install",
                              {"packages": [payload]}))

    def handle_remove_pip(self, payload):
        LOG.info(f"pip package to uninstall: {payload}")
        self.bus.emit(Message("ovos.pip.uninstall",
                              {"packages": [payload]}))

    def handle_bus(self, payload):
        LOG.info(f"bus msg_type: {payload}")
        self.bus.emit(Message(payload))

    def handle_utt(self, payload):
        LOG.info(f"Utterance: {payload}")
        self.bus.emit(Message("recognizer_loop:utterance",
                              {"utterances": [payload]}))

    def handle_wifi_ssid(self, payload):
        LOG.info(f"Wifi AP: {payload}")
        self._ssid = payload

    def handle_speak(self, payload):
        LOG.info(f"Speak: {payload}")
        self.bus.emit(Message("speak", {"utterance": payload}))

    def handle_json(self, payload):
        LOG.info(f"JSON: {payload}")
        try:
            msg = Message.deserialize(payload)
            self.bus.emit(msg)
        except:
            LOG.exception("failed to deserialize message")
            snd = Configuration().get("sounds", {}).get("json_error", "snd/error.mp3")
            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))

    def handle_wifi_pswd(self, payload):
        if not self._ssid:
            LOG.error("received wifi password but wifi SSID not set! ignoring")
            snd = Configuration().get("sounds", {}).get("wifi_error", "snd/error.mp3")
            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))
            return

        if not payload:
            LOG.info("wifi is open, no password")
            data = {"connection_name": self._ssid}
            self.bus.emit(Message("ovos.phal.nm.connect.open.network", data))
        else:
            LOG.info(f"Wifi PSWD {payload}")
            data = {"connection_name": self._ssid, "password": payload}
            self.bus.emit(Message("ovos.phal.nm.connect", data))

        self._ssid = None

    def monitor_thread(self):
        child = pexpect.spawn(self.binpath)
        marker = "Received sound data successfully: "
        while True:
            try:
                txt = child.readline().decode("utf-8").strip()
                if txt and self.debug:
                    LOG.debug(txt)
                if txt.startswith(marker):
                    payload = txt.split(marker)[-1][1:-1]
                    for opcode, handler in self.OPCODES.items():
                        if payload.startswith(opcode):
                            p = payload.split(opcode, 1)[-1]
                            if self.user_enabled:
                                handler(p)
                            else:
                                LOG.debug("ignoring ggwave payload, user did not enable ggwave")
                            break
                    else:
                        LOG.debug(f"invalid ggwave payload: {payload}")
            except pexpect.exceptions.EOF:
                # exited
                LOG.error("Exited ggwave-rx process")
                break
            except pexpect.exceptions.TIMEOUT:
                # nothing happened for a while
                pass
            except KeyboardInterrupt:
                break


if __name__ == "__main__":
    from ovos_utils import wait_for_exit_signal

    binary = "/home/miro/PycharmProjects/ovos-audio-transformer-plugin-ggwave/ggwave/build/bin/ggwave-rx"

    gg = GGWavePlugin({"binary": binary})

    wait_for_exit_signal()

==> ovos_audio_transformer_plugin_ggwave-0.0.3a1-py3-none-any.whl: ovos_audio_transformer_plugin_ggwave/__init__.py <==
import datetime
import threading

import pyaudio  # TODO ditch me
from ovos_config import Configuration
from ovos_plugin_manager.templates.transformers import AudioTransformer
from ovos_utils import create_daemon
from ovos_utils.log import LOG
from ovos_utils.messagebus import Message

import ggwave



class GGWavePlugin(AudioTransformer):

    def __init__(self, config=None):
        config = config or {}
        super().__init__("ovos-audio-transformer-plugin-ggwave", 10, config)

        # TODO - individual config to enable/disable each
        self.OPCODES = {
            "SSID:": self.handle_wifi_ssid,
            "PSWD:": self.handle_wifi_pswd,
            "UTT:": self.handle_utt,
            "SPEAK:": self.handle_speak,
            "JSON:": self.handle_json,
            "BUS:": self.handle_bus,
            "GHS:": self.handle_skill,
            "PIP:": self.handle_pip,
            "RMPIP:": self.handle_remove_pip
        }
        self._ssid = None
        self.user_enabled = self.config.get("start_enabled", False)
        self.ggwave = ggwave.init()
        self._stop = threading.Event()

    def bind(self, bus=None):
        """ attach messagebus """
        super().bind(bus)
        # we load the voice interface as part of this plugin
        # the skill interacts only via messagebus
        self.bus.on("ovos.ggwave.enable", self.handle_enable)
        self.bus.on("ovos.ggwave.disable", self.handle_disable)
        self.daemon = create_daemon(self.monitor_thread)

    def handle_enable(self, message: Message):
        self.user_enabled = True
        self.bus.emit(message.forward("ovos.ggwave.enabled"))
        # TODO - dedicated sound
        self.bus.emit(Message("mycroft.audio.play_sound",
                              {"uri": "snd/acknowledge.mp3"}))

    def handle_disable(self, message: Message):
        self.user_enabled = False
        self.bus.emit(message.forward("ovos.ggwave.disabled"))
        # TODO - dedicated sound
        self.bus.emit(Message("mycroft.audio.play_sound",
                              {"uri": "snd/acknowledge.mp3"}))

    def handle_skill(self, payload):
        if not payload.startswith("https://github.com/"):
            payload = f"https://github.com/{payload}"
        LOG.info(f"github skill to install: {payload}")
        self.bus.emit(Message("ovos.skills.install", {"url": payload}))

    def handle_pip(self, payload):
        LOG.info(f"pip package to install: {payload}")
        self.bus.emit(Message("ovos.pip.install",
                              {"packages": [payload]}))

    def handle_remove_pip(self, payload):
        LOG.info(f"pip package to uninstall: {payload}")
        self.bus.emit(Message("ovos.pip.uninstall",
                              {"packages": [payload]}))

    def handle_bus(self, payload):
        LOG.info(f"bus msg_type: {payload}")
        self.bus.emit(Message(payload))

    def handle_utt(self, payload):
        LOG.info(f"Utterance: {payload}")
        self.bus.emit(Message("recognizer_loop:utterance",
                              {"utterances": [payload]}))

    def handle_wifi_ssid(self, payload):
        LOG.info(f"Wifi AP: {payload}")
        self._ssid = payload
        snd = Configuration().get("sounds", {}).get("wifi_ap")
        if snd:  # no sound by default
            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))

    def handle_speak(self, payload):
        LOG.info(f"Speak: {payload}")
        self.bus.emit(Message("speak", {"utterance": payload}))

    def handle_json(self, payload):
        LOG.info(f"JSON: {payload}")
        try:
            msg = Message.deserialize(payload)
            self.bus.emit(msg)
        except:
            LOG.exception("failed to deserialize message")
            snd = Configuration().get("sounds", {}).get("json_error", "snd/error.mp3")
            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))

    def handle_wifi_pswd(self, payload):
        if not self._ssid:
            LOG.error("received wifi password but wifi SSID not set! ignoring")
            snd = Configuration().get("sounds", {}).get("wifi_error", "snd/error.mp3")
            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))
            return

        if not payload:
            LOG.info("wifi is open, no password")
            data = {"connection_name": self._ssid}
            self.bus.emit(Message("ovos.phal.nm.connect.open.network", data))
        else:
            LOG.info(f"Wifi PSWD {payload}")
            data = {"connection_name": self._ssid, "password": payload}
            self.bus.emit(Message("ovos.phal.nm.connect", data))

        self._ssid = None

    def on_audio(self, audio_data):
        """ Take any action you want, audio_data is a non-speech chunk
        """
        # TODO - remove thread and check ggwave audio here instead!
        # audio_data from listener is usually 16000 sample rate
        # not sure if ggwave requires 1024 chunk size?
        return audio_data

    def monitor_thread(self):
        p = pyaudio.PyAudio()

        stream = p.open(format=pyaudio.paFloat32, channels=1, rate=48000, input=True, frames_per_buffer=1024)

        try:
            while not self._stop.is_set():
                data = stream.read(1024, exception_on_overflow=False)
                res = ggwave.decode(self.ggwave, data)
                if (not res is None):
                    try:
                        payload = res.decode("utf-8")
                        snd = Configuration().get("sounds", {}).get("ggwave_success", "snd/acknowledge.mp3")
                        if snd:
                            # no sound by default
                            self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))

                        for opcode, handler in self.OPCODES.items():
                            if payload.startswith(opcode):
                                p = payload.split(opcode, 1)[-1]
                                if self.user_enabled:
                                    handler(p)
                                else:
                                    LOG.info("ignoring ggwave payload, user did not enable ggwave")
                                break
                        else:
                            LOG.debug(f"invalid ggwave payload: {payload}")
                            snd = Configuration().get("sounds", {}).get("ggwave_error")
                            if snd:
                                # no sound by default
                                self.bus.emit(Message("mycroft.audio.play_sound", {"uri": snd}))
                    except:
                        pass
        except KeyboardInterrupt:
            pass

        stream.stop_stream()
        stream.close()
        p.terminate()

    def default_shutdown(self):
        """ perform any shutdown actions """
        self._stop.set()
        ggwave.free(self.ggwave)


def launch_cli():
    from ovos_utils import wait_for_exit_signal
    from ovos_bus_client.util import get_mycroft_bus

    gg = GGWavePlugin({"start_enabled": True})

    bus = get_mycroft_bus()
    gg.bind(bus)

    wait_for_exit_signal()  # wait for CTRl+C


if __name__ == "__main__":
    launch_cli()
